precision highp float;
precision highp sampler2D;
uniform sampler2D InputBuffer;
uniform sampler2D GradBuffer;
uniform sampler2D NoiseMap;
uniform ivec2 size;
uniform vec2 mapsize;
uniform int yOffset;
out float Output;

#define SIGMA 10.0
#define BSIGMA 0.1
#define MSIZE 7
#define KSIZE (MSIZE-1)/2
#define TRANSPOSE 1
#define INSIZE 1,1
#define NRcancell (0.90)
#define NRshift (+0.6)
#define maxNR (7.)
#define minNR (0.2)
#define NOISES 0.0
#define NOISEO 0.0
#define INTENSE 1.0
#define PI 3.1415926535897932384626433832795
#import gaussian
float lum(in vec4 color) {
    return length(color.xyz);
}

void main() {
    ivec2 xy = ivec2(gl_FragCoord.xy);
    xy+=ivec2(0,yOffset);
    float cin = float(texelFetch(InputBuffer, xy, 0).r);
    ivec2 shift = xy%2;
    ivec2 mvx = ivec2(2,0);
    ivec2 mvy = ivec2(0,2);
    if(shift.x+shift.y ==  1){
        mvx = ivec2(-1,1);
        mvy = ivec2(1,1);
    }

    float noisefactor = texture(NoiseMap, vec2(xy)/vec2(INSIZE)).g;
    {
        float final_colour = float(0.0);
        float sigX = 3.5;
        float sigY = sqrt(noisefactor*NOISES + NOISEO);
        //sigY = max(0.01,sigY);
        //create the 1-D kernel
        float Z = 0.0;
        vec2 dxy;
        vec2 dxyabs;
        float sum = 0.0001;
        vec2 baseGrad = texelFetch(GradBuffer, xy, 0).rg;
        for (int i=-KSIZE; i <= KSIZE; i++){
            float k0 = pdf(float(i)/sigX);
            for (int j=-KSIZE; j <= KSIZE; j++){
                    vec2 temp = texelFetch(GradBuffer, xy+ivec2(-1,1)*i+ivec2(1,1)*j, 0).rg;
                    float k = pdf(float(j)/sigX)*k0*pdf((temp-baseGrad)/sigY);
                    temp *= k;
                    dxy+=temp.x*vec2(-1.0,1.0)+temp.y*vec2(1.0,1.0);
                    sum+=k;
            }
        }
        dxy/=sum;
        //dxy = dxy.x*vec2(-1.0,1.0) + dxy.y*vec2(1.0,1.0);

        float angle = atan(dxy.y,dxy.x)+PI/2.0;
        float cc;
        float factor;
        float sino = sin(angle);
        sino*=sino;
        float coso = cos(angle);
        coso*=coso;
        float sin2o = sin(angle*2.0);
        vec2 sigo = vec2(3.5*1.5,3.5/4.35);
        //vec2 sigo = vec2(25.0*sqrt(sigY),7.0*sqrt(sigY));
        //sigo.y = sKernel/sigo.x;
        //Improve energy
        //sigo*=min(sigY*30.1,1.0);
        sigo*=sigo;
        sigo*=1.0+sigY;

        float a = (coso)/(2.0*sigo.x) + (sino)/(2.0*sigo.y);
        float b = -(sin2o)/(4.0*sigo.x) + (sin2o)/(4.0*sigo.y);
        float c = (sino)/(2.0*sigo.x) + (coso)/(2.0*sigo.y);
        //read out the texels
        for (int i=-KSIZE; i <= KSIZE; ++i)
        {
            for (int j=-KSIZE; j <= KSIZE; ++j)
            {
                ivec2 xyc = i*mvx+j*mvy;
                cc = float(texelFetch(InputBuffer, xy+xyc, 0).r);
                factor = pdf((cc-cin)/sigY)/
                fastExp(
                (a*float(xyc.x*xyc.x) +
                2.0*b*float(xyc.x*xyc.y) +
                c*float(xyc.y*xyc.y))
                //float(xyc.x*xyc.x)+float(xyc.y*xyc.y)
                );

                Z += factor;
                final_colour += factor*cc;
            }
        }

        if (Z < 0.0001f) {
            Output = cin;
        } else {
            Output = float(clamp(final_colour/Z,0.0,1.0));
        }

    }
}